import numpy as np
import sys
import matplotlib.pyplot as plt
import matplotlib
import math
import itertools


def draw(t_cpu, t_fpga, fig_path=None):
    plt.rcParams["font.size"] = 9
    plt.rcParams["font.weight"] = 'regular'
    plt.rcParams["font.family"] = 'serif'
    plt.rcParams["font.serif"] = 'Linux Libertine O'

    lw= 0.75
    ms= 8

    thread_num = np.array([1, 2, 4, 8, 16, 24, 32, 48])

    fig, ax = plt.subplots(figsize=(3.14,2.5))
    ax.set_xlim(1, 48)
    ax.set_ylim(0, 22.5)

    total_entry = 5120000
    col_sel = (1, 5, 9)
    link_len = np.array([1, 16, 128])

    cpu_thr = total_entry*128/t_cpu[:, col_sel]/1000
    fpga_thr = total_entry*128/t_fpga[:, col_sel]/1000

    marker = itertools.cycle(('+', '.', '*', 'v', '^', 's', '<', '>'))

    cpu_curves=[]
    for ii in range(len(col_sel)):
        label_name = 'CPU L=' + str(link_len[ii])
        cpu_curves+= \
            ax.plot(thread_num, cpu_thr[:, ii], label=label_name,
                    marker=next(marker),markerfacecolor='none',
                    linewidth=lw, markersize=ms, clip_on=False)

    fpga_curves=[]
    for ii in range(len(col_sel)):
        label_name = 'FPGA L=' + str(link_len[ii])
        fpga_curves+= \
            ax.plot(thread_num, fpga_thr[:, ii], label=label_name,
                    marker=next(marker), markerfacecolor='none',
                    linewidth=lw, markersize=ms, clip_on=False)

    cpu_labels= [l.get_label() for l in cpu_curves]
    cpu_legend= \
        ax.legend(cpu_curves, cpu_labels, loc='upper left',
                  ncol=1, prop={"size": 8}, frameon=False)
    ax.add_artist(cpu_legend)

    fpga_labels= [l.get_label() for l in fpga_curves]
    ax.legend(fpga_curves, fpga_labels, loc=(0.55,0.0),
              ncol=1, prop={"size": 8}, frameon=False)

    ax.set_xlabel('Threads')
    ax.set_ylabel('DRAM BW (GB/s)')
    ax.grid(linestyle='--', linewidth=0.5)
    ax.set_xticks(np.array([1,8,16,32, 48]))

    axr = ax.twinx()
    ybottom, ytop = ax.get_ylim()
    axr.set_ylim(ybottom/128*1000,ytop/128*1000)
    axr.set_ylabel('Scan rate (M item/s)',
                   rotation=-90, rotation_mode='anchor',
                   ha='center', va='baseline')

    fig.show()
    fig.savefig(fig_path, bbox_inches='tight', pad_inches = 0)


def main():
    data_cpu_in = sys.argv[1]
    data_fpga_in = sys.argv[2]
    fig_name = sys.argv[3]

    t_cpu =  np.genfromtxt(data_cpu_in, delimiter=',')[1:,:].astype(int)
    t_fpga =  np.genfromtxt(data_fpga_in, delimiter=',')[1:,:].astype(int)

    draw(t_cpu, t_fpga, fig_name)


if __name__ == '__main__':
    main()
