import numpy as np
import sys
import matplotlib.pyplot as plt
import matplotlib.ticker as mtick
import math
import csv
import itertools



def draw(t_l1, t_l2, fig_path=None):
    plt.rcParams["font.size"] = 9
    plt.rcParams["font.weight"] = 'regular'
    plt.rcParams["font.family"] = 'serif'
    plt.rcParams["font.serif"] = 'Linux Libertine O'

    lw= 0.75
    ms= 8

    selectivity = 0.1
    num_reuse = t_l1[:, 0]
    num_items = 5120000 * selectivity
    num_query = num_items * (num_reuse + 1)

    fig, ax = plt.subplots(figsize=(3.14,1.5))
    ax.set_ylim(0, 80)
    ax.set_xlim(0, 128)
    ax.set_xticks(np.array([0, 16, 32, 64, 128]))

    x = num_reuse
    y_query_per_sec_l1 = num_query / t_l1[:, 1] # Mega query per second (time is in micro second)
    y_query_per_sec_l2 = num_query / t_l2[:, 1]

    marker = itertools.cycle(('+', '.', '*', 'v', '^', 's', '<', '>'))

    label_name = "Rate w/L1 hits"
    color = next(ax._get_lines.prop_cycler)['color']
    L1_curve= \
        ax.plot(x, y_query_per_sec_l1, color=color, label=label_name,
                marker=next(marker), markerfacecolor='none',
                linewidth=lw, markersize=ms, clip_on=False)
    label_name = "Rate w/L2 hits"
    color = next(ax._get_lines.prop_cycler)['color']
    L2_curve= \
        ax.plot(x, y_query_per_sec_l2, color=color, label=label_name,
                marker=next(marker), markerfacecolor='none',
                linewidth=lw, markersize=ms, clip_on=False)

    ax.set_xlabel('Cache size/Reuse distance')
    ax.set_ylabel('Results (M/s)')
    ax.grid(linestyle='--', linewidth=0.5)

    axr = ax.twinx()
    axr.set_ylim(0,100)
    l2_miss_rate = t_l2[:,2] / num_query
    color = next(ax._get_lines.prop_cycler)['color']
    label_name = "L2 miss rate"
    L2_miss_curve= \
        axr.plot(x, l2_miss_rate*100, color=color, label=label_name,
                 marker=next(marker), markerfacecolor='none',
                 linewidth=lw, markersize=ms, clip_on=False)
    axr.set_ylabel('L2 miss rate (%)', rotation=-90, rotation_mode='anchor',
                   ha='center', va='baseline')

    curves= L1_curve + L2_curve + L2_miss_curve
    labels= [l.get_label() for l in curves]
    ax.legend(curves, labels, loc=(0.55,0.30), ncol=1,
              prop={"size" : 8}, frameon=False, fancybox=False)

    fig.show()
    fig.savefig(fig_path, bbox_inches='tight', pad_inches = 0)


def main():
    fname_l1 = sys.argv[1]
    fname_l2 = sys.argv[2]
    fig_name = sys.argv[3]
    # with open(fname_l1, 'r') as f_csv:
    #     csv_rd = csv.reader(f_csv, delimiter = ',')
    #     title = next(csv_rd)
    t_l1 = np.genfromtxt(fname_l1, delimiter=',')[1:,:].astype(int)
    t_l2 = np.genfromtxt(fname_l2, delimiter=',')[1:, :].astype(int)

    draw(t_l1, t_l2, fig_name)

if __name__ == '__main__':
    main()
