import numpy as np
import sys
import matplotlib.pyplot as plt
import matplotlib
import math
import itertools
from util import adjustFigAspect


def draw(t_cpu, t_fpga, fig_path=None):
    plt.rcParams["font.size"] = 9
    plt.rcParams["font.weight"] = 'normal'
    plt.rcParams["font.family"] = 'serif'
    plt.rcParams["font.serif"] = 'Linux Libertine O'

    lw= 0.75
    ms= 8

    thread_num = np.array([1, 2, 4, 8, 16, 24, 32, 48])

    fig= plt.figure(figsize=(3.14,2.5))
    gs= fig.add_gridspec(2, 1, hspace=0.1, height_ratios=[2,1])
    ax_top, ax_bot= gs.subplots(sharex=True)

    total_entry = 5120000
    col_sel = (1, 2, 6)
    selectivity = np.array([0.01, 0.1, 1.0])

    cpu_thr = total_entry*128/t_cpu[:, col_sel]/1000
    fpga_thr = total_entry*128/t_fpga[:, col_sel]/1000

    cpu_query = total_entry*selectivity/t_cpu[:, col_sel]
    fpga_query = total_entry*selectivity/t_fpga[:, col_sel]

    ax_top.set_ylim(0, 35)
    ax_bot.set_ylim(0, 60)
    ax_bot.set_xlim(1, 48)

    marker = itertools.cycle(('+', '.', '*', 'v', '^', 's', '<', '>'))

    for ii in range(len(col_sel)):
        label_name = 'CPU ' + str(int(selectivity[ii]*100)) + '%'
        this_marker= next(marker)
        ax_top.plot(thread_num, cpu_thr[:, ii], label=label_name,
                    marker=this_marker, markerfacecolor='none',
                    linewidth=lw, markersize=ms, clip_on=False)
        ax_bot.plot(thread_num, cpu_query[:, ii], label=label_name,
                    marker=this_marker, markerfacecolor='none',
                    linewidth=lw, markersize=ms)

    for ii in range(len(col_sel)):
        label_name = 'FPGA ' + str(int(selectivity[ii] * 100)) + '%'
        this_marker= next(marker)
        ax_top.plot(thread_num, fpga_thr[:, ii], label=label_name,
                    marker=this_marker, markerfacecolor='none',
                    linewidth=lw, markersize=ms, clip_on=False)
        ax_bot.plot(thread_num, fpga_query[:, ii], label=label_name,
                    marker=this_marker, markerfacecolor='none',
                    linewidth=lw, markersize=ms)

    ax_top.label_outer()
    ax_bot.label_outer()

    ax_top.legend(loc='best', ncol=2, prop={"size": 8}, frameon=False)
    ax_bot.set_xlabel('Threads')
    ax_top.set_ylabel('DRAM BW (GB/s)')
    ax_bot.set_ylabel('Results (M/s)')
    ax_top.grid(linestyle='--', linewidth=0.5)
    ax_bot.grid(linestyle='--', linewidth=0.5)
    ax_bot.set_xticks(np.array([1,8,16,32, 48]))

    axr_top = ax_top.twinx()
    ybottom, ytop = ax_top.get_ylim()
    axr_top.set_ylim(ybottom/128*1000,ytop/128*1000)
    axr_top.set_ylabel('Scan rate (M item/s)', rotation=-90,
                       rotation_mode='anchor', ha='center', va='baseline')

    fig.show()
    fig.savefig(fig_path, bbox_inches='tight', pad_inches = 0)


def main():
    data_cpu_in = sys.argv[1]
    data_fpga_in = sys.argv[2]
    fig_name = sys.argv[3]

    t_cpu =  np.genfromtxt(data_cpu_in, delimiter=',')[1:,:].astype(int)
    t_fpga =  np.genfromtxt(data_fpga_in, delimiter=',')[1:,:].astype(int)

    draw(t_cpu, t_fpga, fig_name)


if __name__ == '__main__':
    main()
