import numpy as np
import sys
import matplotlib.pyplot as plt
import matplotlib
import math
import itertools
from util import adjustFigAspect


def draw(t_cpu, t_fpga, fig_path=None):
    plt.rcParams["font.size"] = 10
    plt.rcParams["font.weight"] = 'bold'

    thread_num = np.array([1, 2, 4, 8, 16, 24, 32, 48])

    fig, ax = plt.subplots()

    total_entry = 5120000
    col_sel = (1, 2)
    selectivity = np.array([0.01, 0.1])

    cpu_query = total_entry*selectivity/t_cpu[:, col_sel]
    fpga_query = total_entry*selectivity/t_fpga[:, col_sel]

    marker = itertools.cycle(('+', '.', '*', 'v', '^', 's', '<', '>'))

    for ii in range(len(col_sel)):
        label_name = 'CPU (' + str(int(selectivity[ii] * 100)) + '% match)'
        ax.plot(thread_num, cpu_query[:, ii], label=label_name, marker=next(marker), markerfacecolor='none', linewidth=1, markersize=15)

    for ii in range(len(col_sel)):
        label_name = 'FPGA (' + str(int(selectivity[ii] * 100)) + '% match)'
        ax.plot(thread_num, fpga_query[:, ii], label=label_name, marker=next(marker), markerfacecolor='none', linewidth=1, markersize=10)

    # ax.legend(loc='best', ncol=1, prop={"weight": 'bold', "size": 10}, edgecolor='black', fancybox=False, framealpha=0.3)
    ax.set_xlabel('Number of Thread', fontweight='bold')
    ax.set_ylabel('Million Queries per Second', fontweight='bold')
    ax.grid(linestyle='--', linewidth=0.5)
    ax.set_xticks(np.array([1,8,16,32, 48]))

    adjustFigAspect(fig, aspect=0.6)
    fig.show()
    fig.savefig(fig_path, bbox_inches='tight', pad_inches = 0)


def main():
    data_cpu_in = sys.argv[1]
    data_fpga_in = sys.argv[2]
    fig_name = sys.argv[3]

    t_cpu =  np.genfromtxt(data_cpu_in, delimiter=',')[1:,:].astype(int)
    t_fpga =  np.genfromtxt(data_fpga_in, delimiter=',')[1:,:].astype(int)

    draw(t_cpu, t_fpga, fig_name)


if __name__ == '__main__':
    main()
